#ifndef __ist_TaskScheduler__
#define __ist_TaskScheduler__

#ifndef _WIN32_WINNT
  #define _WIN32_WINNT 0x0500
  #define WINVER 0x0500
#endif

#include <EASTL/vector.h>
#include <EASTL/list.h>
#include <boost/thread.hpp>
#include <boost/smart_ptr.hpp>
#include <boost/interprocess/detail/atomic.hpp>

namespace ist {

// XbhɖOݒ肵܂BfobK环ʂ₷Ȃ܂B 
// ̑Xbhɑ΂Ă͎s邱ƂAXbhgɂ̂]܂łB
void SetThreadName(uint32_t thread_id, const char *name);
void SetThreadName(const char *name);


class SpinLock
{
private:
    int m_lock;

public:
    SpinLock() : m_lock(0) { }

    void lock()
    {
        while(::InterlockedCompareExchange((volatile LONG*)&m_lock, 1, 0) != 0) {
        }
    }

    void unlock()
    {
        ::InterlockedExchange((volatile LONG*)&m_lock, 0);
    }
};


class Task;
typedef Task* TaskPtr;


namespace impl
{
    class TaskThread;
    class TaskQueue;
} // namespace impl



// todo: D揇
class Task
{
friend class TaskScheduler;
friend class impl::TaskQueue;
friend class impl::TaskThread;
private:
    bool m_finished;

private:
    void beforeExec();
    void afterExec();

public:
    Task();
    virtual ~Task();
    bool isFinished() const;

    virtual void exec()=0;
};




namespace impl
{

    class TaskQueue;
    class TaskThread;


} // namespace impl



class TaskScheduler
{
private:
    typedef boost::shared_ptr<impl::TaskThread> thread_ptr;
    typedef stl::vector<thread_ptr> thread_cont;
    typedef boost::scoped_ptr<impl::TaskQueue> queue_ptr;

    static TaskScheduler *s_instance;
    queue_ptr m_task_queue;
    thread_cont m_threads;

private:
    TaskScheduler();
    ~TaskScheduler();
    void initialize(size_t num_thread);

public:
    /// num_thread: XbhB0 ̏ꍇ CPU ̐ɎB 
    static void initializeSingleton(size_t num_thread=0);
    /// ݏ̃^XN̊҂ĔjB(^XNL[ɂȂ̂҂Ȃ) 
    static void finalizeSingleton();
    static TaskScheduler* getInstance();

    /// s҂^XN΂BȂ΋UԂB 
    static bool wait();
    /// w^XN̊҂Bs҂^XNꍇAĂяoXbh͑҂Ăԃ^XNɉB 
    /// ^XNԂő݂ɑ҂悤ȃV`G[VƉiv~\̂ŒӁB 
    static void waitFor(TaskPtr task);
    static void waitFor(TaskPtr tasks[], size_t num);
    /// w^XN̊҂BwaitFor() ƈႢAĂяoXbh̓^XNɉȂB
    static void waitExclusive(TaskPtr task);
    static void waitExclusive(TaskPtr tasks[], size_t num);
    /// S^XN̊҂Bs҂^XNꍇAĂяoXbh͑҂Ăԃ^XNɉB 
    /// ^XNĂԂƉiv~̂ł_B 
    static void waitForAll();

    /// ^XÑXPW[OsB 
    static void push(TaskPtr task);
    static void push(TaskPtr tasks[], size_t num);
    static void push_front(TaskPtr task);
    static void push_front(TaskPtr tasks[], size_t num);

    static size_t getThreadCount();
    static boost::thread::id getThreadId(size_t i);


    /// p
    impl::TaskQueue* getTaskQueue();
};




// ȉA[eBeBn


// q/`FC^XN
class ChainedTask : public Task
{
protected:
    stl::vector<TaskPtr> m_children;
    stl::vector<TaskPtr> m_chain;

public:
    void clear()
    {
        m_children.clear();
        m_chain.clear();
    }

    void appendChild(TaskPtr p) { m_children.push_back(p); }
    void appendChain(TaskPtr p) { m_chain.push_back(p); }

    void exec()
    {
        TaskScheduler& scheduler = *TaskScheduler::getInstance();
        scheduler.push(&m_children[0], m_children.size());
        scheduler.waitFor(&m_children[0], m_children.size());
        scheduler.push(&m_chain[0], m_chain.size());
    }
};


} // namespace ist
#endif // __ist_TaskScheduler__ 
